@@ -122,6 +122,7 @@ DEFINE_TEST(test_compat_solaris_pax_sparse)
 DEFINE_TEST(test_compat_solaris_tar_acl)
 DEFINE_TEST(test_compat_star_acl_posix1e)
 DEFINE_TEST(test_compat_star_acl_nfs4)
+DEFINE_TEST(test_compat_tar_directory)
 DEFINE_TEST(test_compat_tar_hardlink)
 DEFINE_TEST(test_compat_uudecode)
 DEFINE_TEST(test_compat_uudecode_large)
@@ -206,15 +207,15 @@ DEFINE_TEST(test_read_format_ar)
 DEFINE_TEST(test_read_format_cab)
 DEFINE_TEST(test_read_format_cab_filename)
 DEFINE_TEST(test_read_format_cpio_afio)
+DEFINE_TEST(test_read_format_cpio_bin)
+DEFINE_TEST(test_read_format_cpio_bin_Z)
 DEFINE_TEST(test_read_format_cpio_bin_be)
 DEFINE_TEST(test_read_format_cpio_bin_bz2)
-DEFINE_TEST(test_read_format_cpio_bin)
 DEFINE_TEST(test_read_format_cpio_bin_gz)
 DEFINE_TEST(test_read_format_cpio_bin_le)
 DEFINE_TEST(test_read_format_cpio_bin_lzip)
 DEFINE_TEST(test_read_format_cpio_bin_lzma)
 DEFINE_TEST(test_read_format_cpio_bin_xz)
-DEFINE_TEST(test_read_format_cpio_bin_Z)
 DEFINE_TEST(test_read_format_cpio_filename_eucJP_UTF8)
 DEFINE_TEST(test_read_format_cpio_filename_UTF8_eucJP)
 DEFINE_TEST(test_read_format_cpio_filename_UTF8_UTF8_jp)
@@ -233,9 +234,9 @@ DEFINE_TEST(test_read_format_cpio_filename_KOI8R_CP1251)
 DEFINE_TEST(test_read_format_cpio_filename_UTF8_CP1251)
 DEFINE_TEST(test_read_format_cpio_odc)
 DEFINE_TEST(test_read_format_cpio_svr4_bzip2_rpm)
-DEFINE_TEST(test_read_format_cpio_svr4c_Z)
 DEFINE_TEST(test_read_format_cpio_svr4_gzip)
 DEFINE_TEST(test_read_format_cpio_svr4_gzip_rpm)
+DEFINE_TEST(test_read_format_cpio_svr4c_Z)
 DEFINE_TEST(test_read_format_empty)
 DEFINE_TEST(test_read_format_gtar_filename_eucJP_UTF8)
 DEFINE_TEST(test_read_format_gtar_filename_CP866_KOI8R)
@@ -250,20 +251,20 @@ DEFINE_TEST(test_read_format_gtar_gz)
 DEFINE_TEST(test_read_format_gtar_lzma)
 DEFINE_TEST(test_read_format_gtar_sparse)
 DEFINE_TEST(test_read_format_gtar_sparse_skip_entry)
+DEFINE_TEST(test_read_format_iso_Z)
+DEFINE_TEST(test_read_format_iso_multi_extent)
+DEFINE_TEST(test_read_format_iso_xorriso)
 DEFINE_TEST(test_read_format_isojoliet_bz2)
 DEFINE_TEST(test_read_format_isojoliet_long)
 DEFINE_TEST(test_read_format_isojoliet_rr)
 DEFINE_TEST(test_read_format_isojoliet_versioned)
-DEFINE_TEST(test_read_format_iso_multi_extent)
 DEFINE_TEST(test_read_format_isorr_bz2)
 DEFINE_TEST(test_read_format_isorr_ce)
 DEFINE_TEST(test_read_format_isorr_new_bz2)
 DEFINE_TEST(test_read_format_isorr_rr_moved)
-DEFINE_TEST(test_read_format_iso_xorriso)
-DEFINE_TEST(test_read_format_iso_Z)
 DEFINE_TEST(test_read_format_isozisofs_bz2)
-DEFINE_TEST(test_read_format_lha_bugfix_0)
 DEFINE_TEST(test_read_format_lha)
+DEFINE_TEST(test_read_format_lha_bugfix_0)
 DEFINE_TEST(test_read_format_lha_filename)
 DEFINE_TEST(test_read_format_lha_filename_UTF16)
 DEFINE_TEST(test_read_format_mtree)
@@ -274,8 +275,31 @@ DEFINE_TEST(test_read_format_mtree_nomagic_v2_form)
 DEFINE_TEST(test_read_format_mtree_nomagic_v2_netbsd_form)
 DEFINE_TEST(test_read_format_mtree_nonexistent_contents_file)
 DEFINE_TEST(test_read_format_mtree_noprint)
+DEFINE_TEST(test_read_format_mtree_tab)
 DEFINE_TEST(test_read_format_mtree_crash747)
 DEFINE_TEST(test_read_format_pax_bz2)
+DEFINE_TEST(test_read_format_rar_set_format)
+DEFINE_TEST(test_read_format_rar_basic)
+DEFINE_TEST(test_read_format_rar_subblock)
+DEFINE_TEST(test_read_format_rar_noeof)
+DEFINE_TEST(test_read_format_rar_unicode_UTF8)
+DEFINE_TEST(test_read_format_rar_unicode_CP932)
+DEFINE_TEST(test_read_format_rar_compress_normal)
+DEFINE_TEST(test_read_format_rar_multi_lzss_blocks)
+DEFINE_TEST(test_read_format_rar_compress_best)
+DEFINE_TEST(test_read_format_rar_ppmd_lzss_conversion)
+DEFINE_TEST(test_read_format_rar_binary)
+DEFINE_TEST(test_read_format_rar_windows)
+DEFINE_TEST(test_read_format_rar_multivolume)
+DEFINE_TEST(test_read_format_rar_multivolume_skip)
+DEFINE_TEST(test_read_format_rar_sfx)
+DEFINE_TEST(test_read_format_rar_multivolume_stored_file)
+DEFINE_TEST(test_read_format_rar_multivolume_stored_file_skip)
+DEFINE_TEST(test_read_format_rar_multivolume_seek_data)
+DEFINE_TEST(test_read_format_rar_multivolume_seek_multiple_files)
+DEFINE_TEST(test_read_format_rar_multivolume_uncompressed_files)
+DEFINE_TEST(test_read_format_rar_ppmd_use_after_free)
+DEFINE_TEST(test_read_format_rar_ppmd_use_after_free2)
 DEFINE_TEST(test_read_format_rar5_set_format)
 DEFINE_TEST(test_read_format_rar5_stored)
 DEFINE_TEST(test_read_format_rar5_compressed)
@@ -315,35 +339,19 @@ DEFINE_TEST(test_read_format_rar5_distance_overflow)
 DEFINE_TEST(test_read_format_rar5_nonempty_dir_stream)
 DEFINE_TEST(test_read_format_rar5_fileattr)
 DEFINE_TEST(test_read_format_rar5_different_window_size)
+DEFINE_TEST(test_read_format_rar5_window_buf_and_size_desync)
 DEFINE_TEST(test_read_format_rar5_arm_filter_on_window_boundary)
 DEFINE_TEST(test_read_format_rar5_different_solid_window_size)
 DEFINE_TEST(test_read_format_rar5_different_winsize_on_merge)
 DEFINE_TEST(test_read_format_rar5_block_size_is_too_small)
-DEFINE_TEST(test_read_format_rar_set_format)
-DEFINE_TEST(test_read_format_rar_basic)
-DEFINE_TEST(test_read_format_rar_subblock)
-DEFINE_TEST(test_read_format_rar_noeof)
-DEFINE_TEST(test_read_format_rar_unicode_UTF8)
-DEFINE_TEST(test_read_format_rar_unicode_CP932)
-DEFINE_TEST(test_read_format_rar_compress_normal)
-DEFINE_TEST(test_read_format_rar_multi_lzss_blocks)
-DEFINE_TEST(test_read_format_rar_compress_best)
-DEFINE_TEST(test_read_format_rar_ppmd_lzss_conversion)
-DEFINE_TEST(test_read_format_rar_binary)
-DEFINE_TEST(test_read_format_rar_windows)
-DEFINE_TEST(test_read_format_rar_multivolume)
-DEFINE_TEST(test_read_format_rar_multivolume_skip)
-DEFINE_TEST(test_read_format_rar_sfx)
-DEFINE_TEST(test_read_format_rar_multivolume_stored_file)
-DEFINE_TEST(test_read_format_rar_multivolume_stored_file_skip)
-DEFINE_TEST(test_read_format_rar_multivolume_seek_data)
-DEFINE_TEST(test_read_format_rar_multivolume_seek_multiple_files)
-DEFINE_TEST(test_read_format_rar_multivolume_uncompressed_files)
-DEFINE_TEST(test_read_format_rar_ppmd_use_after_free)
-DEFINE_TEST(test_read_format_rar_ppmd_use_after_free2)
+DEFINE_TEST(test_read_format_rar5_sfx)
+DEFINE_TEST(test_read_format_rar5_decode_number_out_of_bounds_read)
+DEFINE_TEST(test_read_format_rar5_bad_window_size_in_multiarchive_file)
+DEFINE_TEST(test_read_format_rar5_read_data_block_uninitialized_offset)
 DEFINE_TEST(test_read_format_rar_encryption_data)
 DEFINE_TEST(test_read_format_rar_encryption_header)
 DEFINE_TEST(test_read_format_rar_encryption_partially)
+DEFINE_TEST(test_read_format_rar_filter)
 DEFINE_TEST(test_read_format_rar_invalid1)
 DEFINE_TEST(test_read_format_raw)
 DEFINE_TEST(test_read_format_tar)
@@ -352,6 +360,7 @@ DEFINE_TEST(test_read_format_tar_empty_filename)
 DEFINE_TEST(test_read_format_tar_empty_pax)
 DEFINE_TEST(test_read_format_tar_empty_with_gnulabel)
 DEFINE_TEST(test_read_format_tar_filename)
+DEFINE_TEST(test_read_format_tar_invalid_pax_size)
 DEFINE_TEST(test_read_format_tbz)
 DEFINE_TEST(test_read_format_tgz)
 DEFINE_TEST(test_read_format_tlz)
@@ -360,7 +369,6 @@ DEFINE_TEST(test_read_format_tz)
 DEFINE_TEST(test_read_format_ustar_filename)
 DEFINE_TEST(test_read_format_warc)
 DEFINE_TEST(test_read_format_xar)
-DEFINE_TEST(test_read_format_zip_utf8_paths)
 DEFINE_TEST(test_read_format_zip)
 DEFINE_TEST(test_read_format_zip_ppmd_one_file)
 DEFINE_TEST(test_read_format_zip_ppmd_one_file_blockread)
@@ -374,6 +382,10 @@ DEFINE_TEST(test_read_format_zip_bzip2_one_file)
 DEFINE_TEST(test_read_format_zip_bzip2_one_file_blockread)
 DEFINE_TEST(test_read_format_zip_bzip2_multi)
 DEFINE_TEST(test_read_format_zip_bzip2_multi_blockread)
+DEFINE_TEST(test_read_format_zip_zstd_one_file)
+DEFINE_TEST(test_read_format_zip_zstd_one_file_blockread)
+DEFINE_TEST(test_read_format_zip_zstd_multi)
+DEFINE_TEST(test_read_format_zip_zstd_multi_blockread)
 DEFINE_TEST(test_read_format_zip_xz_multi)
 DEFINE_TEST(test_read_format_zip_xz_multi_blockread)
 DEFINE_TEST(test_read_format_zip_ppmd8_crash_1)
@@ -384,6 +396,7 @@ DEFINE_TEST(test_read_format_zip_lzma_stream_end)
 DEFINE_TEST(test_read_format_zip_lzma_stream_end_blockread)
 DEFINE_TEST(test_read_format_zip_7z_lzma)
 DEFINE_TEST(test_read_format_zip_7z_deflate)
+DEFINE_TEST(test_read_format_zip_utf8_paths)
 DEFINE_TEST(test_read_format_zip_comment_stored)
 DEFINE_TEST(test_read_format_zip_encryption_data)
 DEFINE_TEST(test_read_format_zip_encryption_header)
@@ -463,8 +476,8 @@ DEFINE_TEST(test_ustar_filename_encoding_EUCJP_CP932)
 DEFINE_TEST(test_ustar_filename_encoding_CP932_UTF8)
 DEFINE_TEST(test_ustar_filenames)
 DEFINE_TEST(test_warn_missing_hardlink_target)
-DEFINE_TEST(test_write_disk_appledouble)
 DEFINE_TEST(test_write_disk)
+DEFINE_TEST(test_write_disk_appledouble)
 DEFINE_TEST(test_write_disk_failures)
 DEFINE_TEST(test_write_disk_fixup)
 DEFINE_TEST(test_write_disk_hardlink)
@@ -473,11 +486,11 @@ DEFINE_TEST(test_write_disk_lookup)
 DEFINE_TEST(test_write_disk_mac_metadata)
 DEFINE_TEST(test_write_disk_no_hfs_compression)
 DEFINE_TEST(test_write_disk_perms)
+DEFINE_TEST(test_write_disk_secure)
 DEFINE_TEST(test_write_disk_secure744)
 DEFINE_TEST(test_write_disk_secure745)
 DEFINE_TEST(test_write_disk_secure746a)
 DEFINE_TEST(test_write_disk_secure746b)
-DEFINE_TEST(test_write_disk_secure)
 DEFINE_TEST(test_write_disk_sparse)
 DEFINE_TEST(test_write_disk_symlink)
 DEFINE_TEST(test_write_disk_times)
@@ -524,22 +537,22 @@ DEFINE_TEST(test_write_format_cpio_odc)
 DEFINE_TEST(test_write_format_gnutar)
 DEFINE_TEST(test_write_format_gnutar_filenames)
 DEFINE_TEST(test_write_format_gnutar_linknames)
-DEFINE_TEST(test_write_format_iso9660_boot)
 DEFINE_TEST(test_write_format_iso9660)
+DEFINE_TEST(test_write_format_iso9660_boot)
 DEFINE_TEST(test_write_format_iso9660_empty)
 DEFINE_TEST(test_write_format_iso9660_filename)
 DEFINE_TEST(test_write_format_iso9660_zisofs)
-DEFINE_TEST(test_write_format_mtree_absolute_path)
 DEFINE_TEST(test_write_format_mtree)
 DEFINE_TEST(test_write_format_mtree_no_leading_dotslash)
+DEFINE_TEST(test_write_format_mtree_absolute_path)
 DEFINE_TEST(test_write_format_mtree_classic)
 DEFINE_TEST(test_write_format_mtree_classic_indent)
 DEFINE_TEST(test_write_format_mtree_fflags)
 DEFINE_TEST(test_write_format_mtree_no_separator)
 DEFINE_TEST(test_write_format_mtree_quoted_filename)
 DEFINE_TEST(test_write_format_pax)
-DEFINE_TEST(test_write_format_raw_b64)
 DEFINE_TEST(test_write_format_raw)
+DEFINE_TEST(test_write_format_raw_b64)
 DEFINE_TEST(test_write_format_shar_empty)
 DEFINE_TEST(test_write_format_tar)
 DEFINE_TEST(test_write_format_tar_empty)
@@ -558,6 +571,7 @@ DEFINE_TEST(test_write_format_zip_winzip_aes256_encryption)
 DEFINE_TEST(test_write_format_zip_compression_store)
 DEFINE_TEST(test_write_format_zip_empty)
 DEFINE_TEST(test_write_format_zip_empty_zip64)
+DEFINE_TEST(test_write_format_zip_size_unset)
 DEFINE_TEST(test_write_format_zip_file)
 DEFINE_TEST(test_write_format_zip_file_zip64)
 DEFINE_TEST(test_write_format_zip_large)
