@@ -736,6 +736,130 @@ DEFINE_TEST(test_read_format_zip_bzip2_multi_blockread)
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
 }
 
+DEFINE_TEST(test_read_format_zip_zstd_one_file)
+{
+	const char *refname = "test_read_format_zip_zstd.zipx";
+	struct archive *a;
+	struct archive_entry *ae;
+
+	assert((a = archive_read_new()) != NULL);
+	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
+		skipping("zstd is not fully supported on this platform");
+		archive_read_close(a);
+		return;
+	}
+	extract_reference_file(refname);
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_zip(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_open_filename(a, refname, 37));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("vimrc", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one(a, ae, 0xBA8E3BAA));
+	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
+}
+
+DEFINE_TEST(test_read_format_zip_zstd_one_file_blockread)
+{
+	const char *refname = "test_read_format_zip_zstd.zipx";
+	struct archive *a;
+	struct archive_entry *ae;
+
+	assert((a = archive_read_new()) != NULL);
+	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
+		skipping("zstd is not fully supported on this platform");
+		archive_read_close(a);
+		return;
+	}
+	extract_reference_file(refname);
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_zip(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_open_filename(a, refname, 37));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("vimrc", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one_using_blocks(a, 13, 0xBA8E3BAA));
+	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
+}
+
+DEFINE_TEST(test_read_format_zip_zstd_multi)
+{
+	const char *refname = "test_read_format_zip_zstd_multi.zipx";
+	struct archive *a;
+	struct archive_entry *ae;
+
+	assert((a = archive_read_new()) != NULL);
+	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
+		skipping("zstd is not fully supported on this platform");
+		archive_read_close(a);
+		return;
+	}
+	extract_reference_file(refname);
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_zip(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_open_filename(a, refname, 37));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("smartd.conf", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one(a, ae, 0x8DD7379E));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("ts.conf", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one(a, ae, 0x7AE59B31));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("vimrc", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one(a, ae, 0xBA8E3BAA));
+
+	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
+}
+
+DEFINE_TEST(test_read_format_zip_zstd_multi_blockread)
+{
+	const char *refname = "test_read_format_zip_zstd_multi.zipx";
+	struct archive *a;
+	struct archive_entry *ae;
+
+	assert((a = archive_read_new()) != NULL);
+	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
+		skipping("zstd is not fully supported on this platform");
+		archive_read_close(a);
+		return;
+	}
+	extract_reference_file(refname);
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_zip(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_open_filename(a, refname, 37));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("smartd.conf", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one_using_blocks(a, 12, 0x8DD7379E));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("ts.conf", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one_using_blocks(a, 13, 0x7AE59B31));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("ZIP 2.0 (zstd)", archive_format_name(a));
+	assertEqualString("vimrc", archive_entry_pathname(ae));
+	assertEqualIntA(a, 0, extract_one_using_blocks(a, 14, 0xBA8E3BAA));
+
+	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
+}
+
 DEFINE_TEST(test_read_format_zip_xz_multi)
 {
 	const char *refname = "test_read_format_zip_xz_multi.zipx";
@@ -998,3 +1122,49 @@ DEFINE_TEST(test_read_format_zip_7z_lzma)
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
 }
+
+DEFINE_TEST(test_read_format_zip_7z_deflate)
+{
+	const char *refname = "test_read_format_zip_7z_deflate.zip";
+	struct archive_entry *ae;
+	struct archive *a;
+	int r;
+
+	assert((a = archive_read_new()) != NULL);
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_filter_all(a));
+	extract_reference_file(refname);
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_zip(a));
+	assertEqualIntA(a, ARCHIVE_OK,
+		archive_read_open_filename(a, refname, 10240));
+	//read first symlink
+	r = archive_read_next_header(a, &ae);
+	if (archive_zlib_version() == NULL) {
+		assertEqualInt(ARCHIVE_FAILED, r);
+		assertEqualString(archive_error_string(a),
+		    "Unsupported ZIP compression method during decompression "
+		    "of link entry (8: deflation)");
+		assert(archive_errno(a) != 0);
+	} else {
+		assertEqualIntA(a, ARCHIVE_OK, r);
+		assertEqualString("libxkbcommon-x11.so.0.0.0",
+			archive_entry_symlink(ae));
+	}
+	assertEqualInt(AE_IFLNK, archive_entry_filetype(ae));
+	//read second symlink
+	r = archive_read_next_header(a, &ae);
+	if (archive_zlib_version() == NULL) {
+		assertEqualInt(ARCHIVE_FAILED, r);
+		assertEqualString(archive_error_string(a),
+		    "Unsupported ZIP compression method during decompression "
+		    "of link entry (8: deflation)");
+		assert(archive_errno(a) != 0);
+	} else {
+		assertEqualIntA(a, ARCHIVE_OK, r);
+		assertEqualString("libxkbcommon-x11.so.0.0.0",
+			archive_entry_symlink(ae));
+	}
+	assertEqualInt(AE_IFLNK, archive_entry_filetype(ae));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_close(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_free(a));
+}
